//------------------------------------------------------------------------------ 
// Copyright (c) 2004 Xilinx, Inc. 
// All Rights Reserved 
//------------------------------------------------------------------------------ 
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /   Vendor: Xilinx 
// \   \   \/    Author: Latha Pillai, Advanced Product Group, Xilinx, Inc.
//  \   \        Filename: FIR_MCF6
//  /   /        Date Last Modified:  June 23, 2004 
// /___/   /\    Date Created: June 23, 2004 
// \   \  /  \ 
//  \___\/\___\ 
// 
//
// Revision History: 
// $Log: $
//------------------------------------------------------------------------------ 
//
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//------------------------------------------------------------------------------ 
/**********************************************************************/
`timescale  1 ns / 1 ps

module FIR_MCF6 (SLICE_CLK, RST, A_IN, B_IN, C_IN, D_IN, E_IN, F_IN,  
               // C1_IN, C2_IN, C3_IN, C4_IN, C5_IN, C6_IN, C7_IN, C8_IN,
                OUT1, OUT2, OUT3, OUT4, OUT5, OUT6);
	
input           SLICE_CLK, RST;
input   [7:0]  A_IN, B_IN, C_IN, D_IN, E_IN, F_IN;
//input   [7:0]  C1_IN, C2_IN, C3_IN, C4_IN, C5_IN, C6_IN, C7_IN, C8_IN;
output  [9:0]  OUT1, OUT2, OUT3, OUT4, OUT5, OUT6;

reg  [7:0]  A_IN_REG, B_IN_REG, C_IN_REG, D_IN_REG, E_IN_REG, F_IN_REG; 
reg  [7:0] 	MUX_OUT1A, MUX_OUT1B, MUX_OUT1C;
reg  [7:0] MUX_OUT2A, MUX_OUT2B, MUX_OUT;
wire [47:0]  SUM_INT1, SUM_INT2, SUM_INT3, SUM_INT4;
wire [47:0]  SUM_INT5, SUM_INT6, SUM_INT7, SUM_INT8;
wire [7:0]  Q_OUT1, Q_OUT2, Q_OUT3, Q_OUT4, Q_OUT5, Q_OUT6,
             Q_OUT7, Q_OUT8;

reg OUTPUT_EN;
reg [5:0] DLY_CNTR;
reg [7:0]  C1_IN, C2_IN, C3_IN, C4_IN, C5_IN, C6_IN, C7_IN, C8_IN;
reg  [9:0]  OUT1, OUT2, OUT3, OUT4, OUT5, OUT6;
reg IN_MUX_SEL1, IN_MUX_SEL2, IN_MUX_SEL3, IN_MUX_SEL4,
    IN_MUX_SEL5, IN_MUX_SEL6;
reg OUT_MUX_SEL1, OUT_MUX_SEL2, OUT_MUX_SEL3, OUT_MUX_SEL4,
    OUT_MUX_SEL5, OUT_MUX_SEL6;

//wire[7:0] OUT_SEL;
reg[9:0] SUM_INT8_REG;

//
//Input mux (6 to 1) select line
//

always @ (posedge SLICE_CLK or posedge RST)
   begin
   if (RST)
       begin IN_MUX_SEL1 <= 1'b0; IN_MUX_SEL2 <= 1'b0; 
             IN_MUX_SEL3 <= 1'b0; IN_MUX_SEL4 <= 1'b0; 
             IN_MUX_SEL5 <= 1'b0; IN_MUX_SEL6 <= 1'b1; end
   else 
       begin 
                IN_MUX_SEL1 <= IN_MUX_SEL6;
                IN_MUX_SEL2 <= IN_MUX_SEL1;
                IN_MUX_SEL3 <= IN_MUX_SEL2;
                IN_MUX_SEL4 <= IN_MUX_SEL3;
                IN_MUX_SEL5 <= IN_MUX_SEL4;
                IN_MUX_SEL6 <= IN_MUX_SEL5;
        end
   end

// store coefficients
always @ (posedge SLICE_CLK or posedge RST)
   begin
   if (RST)
       begin 
             C1_IN <= 8'b0; C2_IN <= 8'b0; 
             C3_IN <= 8'b0; C4_IN <= 8'b0; 
             C5_IN <= 8'b0; C6_IN <= 8'b0; 
             C7_IN <= 8'b0; C8_IN <= 8'b0; 
       end
   else 
       begin 
             C1_IN <= 8'b10101010; 
             C2_IN <= 8'b01010101; 
             C3_IN <= 8'b11110000; 
             C4_IN <= 8'b11001100; 
             C5_IN <= 8'b01111110; 
             C6_IN <= 8'b00110011; 
             C7_IN <= 8'b11111111; 
             C8_IN <= 8'b10101010; 
        end
   end

// register input to match pipe-lining with IN_MUX_SEL signals

always @ (posedge SLICE_CLK or posedge RST)
   begin
   if (RST)
       begin 
       A_IN_REG <= 8'b0; B_IN_REG <= 8'b0;
       C_IN_REG <= 8'b0; D_IN_REG <= 8'b0; 
       E_IN_REG <= 8'b0; F_IN_REG <= 8'b0;
       end
   else 
       begin 
       A_IN_REG <= A_IN; B_IN_REG <= B_IN;
       C_IN_REG <= C_IN; D_IN_REG <= D_IN; 
       E_IN_REG <= E_IN; F_IN_REG <= F_IN;
       end
   end

//
// Input mux.
//

always @ (posedge SLICE_CLK or posedge RST)
   begin
   if (RST)
       begin 
       MUX_OUT1A <= 8'b0; MUX_OUT1B <= 8'b0;
       MUX_OUT1C <= 8'b0; MUX_OUT2A <= 8'b0; 
		 MUX_OUT2B <= 8'b0; MUX_OUT <= 18'b0;
       end
   else 
      begin 
	  case ({IN_MUX_SEL1,IN_MUX_SEL2})
	  2'b01: MUX_OUT1A <= B_IN_REG;
	  2'b10: MUX_OUT1A <= A_IN_REG;
	  default MUX_OUT1A <= 8'b0;
	  endcase
      case ({IN_MUX_SEL3,IN_MUX_SEL4})
	  2'b01: MUX_OUT1B <= D_IN_REG;
	  2'b10: MUX_OUT1B <= C_IN_REG;
	  default MUX_OUT1B <= 8'b0;
	  endcase
      case ({IN_MUX_SEL5,IN_MUX_SEL6})
	  2'b01: MUX_OUT1C <= F_IN_REG;
	  2'b10: MUX_OUT1C <= E_IN_REG;
	  default MUX_OUT1C <= 8'b0;
	  endcase
      //MUX_OUT1A <= (IN_MUX_SEL1 * A_IN_REG) |  (IN_MUX_SEL2 * B_IN_REG); 
      //MUX_OUT1B <= (IN_MUX_SEL3 * C_IN_REG) |  (IN_MUX_SEL4 * D_IN_REG);
      //MUX_OUT1C <= (IN_MUX_SEL5 * E_IN_REG) |  (IN_MUX_SEL6 * F_IN_REG);
      MUX_OUT2A <= (MUX_OUT1A | MUX_OUT1B);
      MUX_OUT2B <=  MUX_OUT1C ; // to match pipe stage
      MUX_OUT <= (MUX_OUT2A | MUX_OUT2B);
      end
   end

// Shift the output of the mux into a 7 deep 18 wide shift register
//Input Tap SRL18. Each SRL18_7 has 18 shift registers each 7 bits deep. 
//One SRL18_7 is used with each DSP48 slice.

SRL18_7 U1 (.CLK(SLICE_CLK), .RST(RST), .D(MUX_OUT), .Q(Q_OUT1));

SRL18_7 U2 (.CLK(SLICE_CLK), .RST(RST), .D(Q_OUT1), .Q(Q_OUT2));

SRL18_7 U3 (.CLK(SLICE_CLK), .RST(RST), .D(Q_OUT2), .Q(Q_OUT3));

SRL18_7 U4 (.CLK(SLICE_CLK), .RST(RST), .D(Q_OUT3), .Q(Q_OUT4));

SRL18_7 U5 (.CLK(SLICE_CLK), .RST(RST), .D(Q_OUT4), .Q(Q_OUT5));

SRL18_7 U6 (.CLK(SLICE_CLK), .RST(RST), .D(Q_OUT5), .Q(Q_OUT6));

SRL18_7 U7 (.CLK(SLICE_CLK), .RST(RST), .D(Q_OUT6), .Q(Q_OUT7));

SRL18_7 U8 (.CLK(SLICE_CLK), .RST(RST), .D(Q_OUT7), .Q(Q_OUT8));

//          
// Slice Chain. Each slice corresponds to 1 Tap
//

FIR_SLICE1 U21 (.CLK(SLICE_CLK), .RST(RST), .A_IN(Q_OUT1), .C_IN(C1_IN),  
                .POUT(), .PCOUT(SUM_INT1));

FIR_SLICE2 U22 (.CLK(SLICE_CLK), .RST(RST), .A_IN(Q_OUT2), .C_IN(C2_IN),  
              .PCIN(SUM_INT1), .POUT(), .PCOUT(SUM_INT2));

FIR_SLICE2 U23 (.CLK(SLICE_CLK), .RST(RST), .A_IN(Q_OUT3), .C_IN(C3_IN),
              .PCIN(SUM_INT2), .POUT(), .PCOUT(SUM_INT3));

FIR_SLICE2 U24 (.CLK(SLICE_CLK), .RST(RST), .A_IN(Q_OUT4), .C_IN(C4_IN), 
              .PCIN(SUM_INT3), .POUT(), .PCOUT(SUM_INT4));

FIR_SLICE2 U25 (.CLK(SLICE_CLK), .RST(RST), .A_IN(Q_OUT5), .C_IN(C5_IN), 
              .PCIN(SUM_INT4), .POUT(), .PCOUT(SUM_INT5));

FIR_SLICE2 U26 (.CLK(SLICE_CLK), .RST(RST), .A_IN(Q_OUT6), .C_IN(C6_IN), 
              .PCIN(SUM_INT5), .POUT(), .PCOUT(SUM_INT6));

FIR_SLICE2 U27 (.CLK(SLICE_CLK), .RST(RST), .A_IN(Q_OUT7), .C_IN(C7_IN), 
              .PCIN(SUM_INT6), .POUT(), .PCOUT(SUM_INT7));

FIR_SLICE2 U28 (.CLK(SLICE_CLK), .RST(RST), .A_IN(Q_OUT8), .C_IN(C8_IN), 
              .PCIN(SUM_INT7), .POUT(SUM_INT8), .PCOUT());


// initial latency counter. The delay is equal to 
//[(number of channels + 1) * number of taps] + 4 pipe stages

always @ (posedge SLICE_CLK or posedge RST)
   begin
   if (RST)
       begin DLY_CNTR <= 6'b0; end
   else 
       begin 
          if (DLY_CNTR < 6'b111100)
             DLY_CNTR <= DLY_CNTR + 1; 
          else DLY_CNTR <= DLY_CNTR;
       end 
   end

//output enable is on after the initial delay
always @ (posedge SLICE_CLK or posedge RST)
   begin
   if (RST)
       OUTPUT_EN <= 1'b0; 
   else //if(DLY_CNTR == 6'b111011)
       OUTPUT_EN <= (DLY_CNTR == 6'b111100) ? 1'b1 : 1'b0;
   end

always @ (posedge SLICE_CLK or posedge RST)
   begin
   if (RST)
       SUM_INT8_REG <= 8'b0; 
   else 
       SUM_INT8_REG <= SUM_INT8;
   end

//
// Output mux
//
always @ (posedge SLICE_CLK or posedge RST)
   begin
   if (RST)
       begin OUT_MUX_SEL1 <= 1'b0; OUT_MUX_SEL2 <= 1'b0; 
             OUT_MUX_SEL3 <= 1'b0; OUT_MUX_SEL4 <= 1'b0; 
             OUT_MUX_SEL5 <= 1'b0; OUT_MUX_SEL6 <= 1'b1; end
   else 
       begin 
                OUT_MUX_SEL1 <= OUT_MUX_SEL6;
                OUT_MUX_SEL2 <= OUT_MUX_SEL1;
                OUT_MUX_SEL3 <= OUT_MUX_SEL2;
                OUT_MUX_SEL4 <= OUT_MUX_SEL3;
                OUT_MUX_SEL5 <= OUT_MUX_SEL4;
                OUT_MUX_SEL6 <= OUT_MUX_SEL5;
        end
   end

//assign OUT_SEL = { OUT_MUX_SEL6, OUT_MUX_SEL5, 
//                    OUT_MUX_SEL4, OUT_MUX_SEL3, OUT_MUX_SEL2, OUT_MUX_SEL1};
//

always @ (posedge SLICE_CLK or posedge RST)
   begin
   if (RST)
       begin OUT1 <= 10'b0; OUT2 <= 10'b0; OUT3 <= 10'b0; OUT4 <= 10'b0;
             OUT5 <= 10'b0; OUT6 <= 10'b0; end
   else 
      if (OUTPUT_EN == 1'b1)
        begin
        OUT1 <= (OUT_MUX_SEL1) ? SUM_INT8_REG[9:0] : 10'b0;
        OUT2 <= (OUT_MUX_SEL2) ? SUM_INT8_REG[9:0] : 10'b0;
        OUT3 <= (OUT_MUX_SEL3) ? SUM_INT8_REG[9:0] : 10'b0;
        OUT4 <= (OUT_MUX_SEL4) ? SUM_INT8_REG[9:0] : 10'b0;
        OUT5 <= (OUT_MUX_SEL5) ? SUM_INT8_REG[9:0] : 10'b0;
        OUT6 <= (OUT_MUX_SEL6) ? SUM_INT8_REG[9:0] : 10'b0;
		end
     else
        begin 
        end
     // end
   end


////////////////////////////////////////////////////////////////////////////////

endmodule



